package com.rh.test;

import org.jbpm.services.task.deadlines.NotificationListener;
import org.jbpm.services.task.deadlines.notifications.impl.NotificationListenerManager;
import org.kie.api.task.model.Group;
import org.kie.api.task.model.OrganizationalEntity;
import org.kie.internal.task.api.TaskModelProvider;
import org.kie.internal.task.api.UserInfo;
import org.kie.internal.task.api.model.*;

import java.util.*;

public class CustomNotificationListener implements org.jbpm.services.task.deadlines.NotificationListener
{
   @Override
   public void onNotification(NotificationEvent event, UserInfo userInfo)
   {
       List<NotificationListener> notificationListeners = NotificationListenerManager.get().getNotificationListeners();
       for (NotificationListener notificationListener : notificationListeners) {
           System.out.println("\tlistener: "+ notificationListener);
       }
       System.out.println("\tNumber of listeners: "+ notificationListeners.size());

       //try {
       //    throw new Exception("stacktrace");
       //} catch (Exception e) {
       //    e.printStackTrace(System.out);
       //}

       EmailNotification notification = (EmailNotification) event.getNotification();
       Map<String, List<OrganizationalEntity>> users = getLanguages(userInfo, notification);

       Set<String> langs = users.keySet();
       for (String language : langs) {
           Language lang = TaskModelProvider.getFactory().newLanguage();
           lang.setMapkey(language);

           Map<? extends Language, ? extends EmailNotificationHeader> headers = notification.getEmailHeaders();
           EmailNotificationHeader emailNotificationHeader = headers.get(lang);
           String from = emailNotificationHeader.getFrom();
           List<OrganizationalEntity> organizationalEntities = users.get(language);
           String subject = emailNotificationHeader.getSubject();
           String body = emailNotificationHeader.getBody();
           System.out.println("Custom notification: " + event.getTask().getName() + "-" + event.getTask().getId());
           System.out.println("from: " + from);
           for (OrganizationalEntity organizationalEntity : organizationalEntities) {
               System.out.println("to: " + organizationalEntity.getId());
           }
           System.out.println("subject: " + subject);
           System.out.println("body: " + body);
       }
   }

    private Map<String, List<OrganizationalEntity>> getLanguages(UserInfo userInfo, EmailNotification notification) {
        List<OrganizationalEntity> entities = new ArrayList<>();
        entities.addAll(notification.getBusinessAdministrators());
        entities.addAll(notification.getRecipients());

        Map<String, List<OrganizationalEntity>> users = new HashMap<>();
        for (OrganizationalEntity entity : entities) {
            if (entity instanceof Group) {
                buildMapByLanguage(users, (Group) entity, userInfo);
            } else {
                buildMapByLanguage(users, entity, userInfo);
            }
        }
        return users;
    }

    protected void buildMapByLanguage(Map<String, List< OrganizationalEntity >> map, Group group, UserInfo userInfo) {
       Iterator<OrganizationalEntity> it = userInfo.getMembersForGroup(group);
       if (it != null) {
           while (it.hasNext()) {
               OrganizationalEntity entity = it.next();
               if (entity instanceof Group) {
                   buildMapByLanguage(map, (Group) entity, userInfo);
               } else {
                   buildMapByLanguage(map, entity, userInfo);
               }
           }
       }
   }

   protected void buildMapByLanguage(Map<String, List<OrganizationalEntity>> map, OrganizationalEntity user, UserInfo userInfo) {
       String language = Optional.ofNullable(userInfo.getLanguageForEntity(user)).orElse("en-UK");
       List<OrganizationalEntity> list = map.computeIfAbsent(language, k -> new ArrayList<>());
       list.add(user);
   }

    @Override
    public String toString() {
        return "Custom listener";
    }
}
